from typing import Any, Callable

from grpc import ServicerContext
from grpc_interceptor import ServerInterceptor


def _get_authorization(meta):
    for item in meta:
        if item.key.lower() == "authorization":
            return item.value


def _make_request(registry):
    from pyramid.scripting import prepare

    return prepare(registry=registry)["request"]


class RequestInterseptor(ServerInterceptor):
    def __init__(self, pyramid_app, extra_environ=None):
        self.pyramid_app = pyramid_app
        self.extra_environ = extra_environ or {}

    def intercept(
        self,
        method: Callable,
        request: Any,
        context: ServicerContext,
        method_name: str,
    ) -> Any:
        pyramid_request = _make_request(self.pyramid_app.registry)
        pyramid_request.environ.update(self.extra_environ)
        auth = _get_authorization(context.invocation_metadata())
        if auth:
            pyramid_request.environ.update({"HTTP_AUTHORIZATION": auth})

        context.pyramid_request = pyramid_request

        return method(request, context)



import logging
from concurrent import futures

import click
import grpc
from pyramid.paster import bootstrap

from pyramid_grpc.main import build_interceptors, serve

logger = logging.getLogger(__name__)


@click.command()
@click.argument("ini_location")
def run(ini_location):
    """Simple program that greets NAME for a total of COUNT times."""

    logger.setLevel(logging.INFO)

    logger.info("Starting server")
    env = bootstrap(ini_location)

    env["registry"]
    app = env["app"]
    env["root"]
    env["request"]
    env["closer"]

    server = grpc.server(
        futures.ThreadPoolExecutor(max_workers=10),
        interceptors=build_interceptors(app),
    )

    serve(app, server)


if __name__ == "__main__":
    run()



from functools import wraps

import grpc
from pyramid.interfaces import (
    ISecurityPolicy,
)

services = []


def config_grpc_call(factory=None, permission=None):
    def decorator(function):
        @wraps(function)
        def wrapper(self, request, context):
            # If permission not set no need to perform any further check!
            if not permission:
                return function(self, request, context)

            pcontext = factory(context.pyramid_request) if factory else context.pyramid_request.context

            policy = context.pyramid_request.registry.queryUtility(ISecurityPolicy)
            if policy.permits(context.pyramid_request, pcontext, permission):
                return function(self, request, context)
            else:
                context.abort(grpc.StatusCode.PERMISSION_DENIED, "Permission Denied")

        return wrapper

    return decorator


def config_grpc_service(function):
    services.append(function)

    def wrapper():
        pass

    return wrapper


def get_services():
    return services



from typing import Any, Callable

from grpc import ServicerContext
from grpc_interceptor import ServerInterceptor


class TransactionInterseptor(ServerInterceptor):
    def __init__(self, pyramid_app, extra_environ=None):
        self.pyramid_app = pyramid_app
        # self.session = pyramid_app.registry["dbsession_factory"]()

    def intercept(
        self,
        method: Callable,
        request: Any,
        context: ServicerContext,
        method_name: str,
    ) -> Any:
        context.pyramid_request.tm.begin()
        try:
            response = method(request, context)
            context.pyramid_request.tm.commit()
        except Exception as e:
            context.pyramid_request.tm.abort()
            raise e
        return response



from pyramid_grpc.decorators import get_services
from pyramid_grpc.interseptors.request import RequestInterseptor


def build_interceptors(pyramid_app):
    return [RequestInterseptor(pyramid_app)]


def configure_server(pyramid_app, grpc_server):
    for func in get_services():
        func(grpc_server, pyramid_app)


def serve(pyramid_app, grpc_server):
    configure_server(pyramid_app, grpc_server)
    port = pyramid_app.registry.settings.get("grpc.port", "50051")
    grpc_server.add_insecure_port(f"[::]:{port}")

    grpc_server.start()
    grpc_server.wait_for_termination()
